CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
63 lines
1.8 KiB
C++
63 lines
1.8 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#pragma once
|
|
|
|
enum class SyncthreadsKind { kDefault, kCount, kAnd, kOr };
|
|
|
|
template <SyncthreadsKind kind> __device__ int Syncthreads(int predicate) {
|
|
if constexpr (kind == SyncthreadsKind::kDefault) {
|
|
__syncthreads();
|
|
return 0;
|
|
} else if constexpr (kind == SyncthreadsKind::kCount) {
|
|
return __syncthreads_count(predicate);
|
|
} else if constexpr (kind == SyncthreadsKind::kAnd) {
|
|
return __syncthreads_and(predicate);
|
|
} else if constexpr (kind == SyncthreadsKind::kOr) {
|
|
return __syncthreads_or(predicate);
|
|
}
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsKernel(int* out) {
|
|
extern __shared__ int shared_mem[];
|
|
|
|
shared_mem[threadIdx.x] = threadIdx.x + 1;
|
|
|
|
Syncthreads<kind>(0);
|
|
|
|
if (threadIdx.x == 0) {
|
|
int sum = 0;
|
|
for (int i = 0; i < blockDim.x; ++i) {
|
|
sum += shared_mem[i];
|
|
}
|
|
out[blockIdx.x] = sum;
|
|
}
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsZeroKernel(int* out) {
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
out[tid] = Syncthreads<kind>(0);
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsOneKernel(int* out) {
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
out[tid] = Syncthreads<kind>(1);
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsOddEvenKernel(int* out) {
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
out[tid] = Syncthreads<kind>(threadIdx.x % 2);
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsNegativeKernel(int* out) {
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
out[tid] = Syncthreads<kind>(-1);
|
|
}
|
|
|
|
template <SyncthreadsKind kind> __global__ void SyncthreadsIdKernel(int* out) {
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
out[tid] = Syncthreads<kind>(threadIdx.x);
|
|
} |